import * as React from "react";
import FormItemSectionLink from "./form-item.section-link";
import FormItemNumberField from "./form-item.number-field";
import FormItemArray from "./form-item.array";
import FormItemTextarea from "./form-item.textarea";
import FormItemSelect from "./form-item.select";
import FormItemChildren from "./form-item.children";
import FormItemCheckbox from "./form-item.checkbox";
import FormItemMapping from "./form-item.mapping";
import { checkIsDifferentSchema, formItemAttributeMapping, formItemMapping, generateExampleData, getInitialOneOfAnyOfState, getOneOfAnyOfSelectOptions, isMapping, isSelect, } from "./form-section.utilities";
import FormOneOfAnyOf from "./form-one-of-any-of";
class FormControl extends React.Component {
    constructor(props) {
        super(props);
        /**
         * Handles the onClick of the section link, changes from controlled to uncontrolled if location is passed
         */
        this.handleSectionLinkClick = (schemaLocation, dataLocation) => {
            this.handleUpdateSection(schemaLocation, dataLocation);
        };
        /**
         * Handles updating the schema to another active oneOf/anyOf schema
         */
        this.handleAnyOfOneOfClick = (activeIndex) => {
            const updatedData = generateExampleData(this.props.schema[this.state.oneOfAnyOf.type][activeIndex], "");
            this.props.onChange(this.props.dataLocation, updatedData);
            this.setState({
                schema: this.props.schema[this.state.oneOfAnyOf.type][activeIndex],
                oneOfAnyOf: {
                    type: this.state.oneOfAnyOf.type,
                    activeIndex,
                },
            });
        };
        this.state = getInitialOneOfAnyOfState(this.props.schema, this.props.data);
    }
    render() {
        return (React.createElement(React.Fragment, null,
            this.renderAnyOfOneOfSelect(),
            this.renderFormItem()));
    }
    /**
     * React lifecycle hook
     */
    componentWillUpdate(nextProps) {
        if (checkIsDifferentSchema(this.props.schema, nextProps.schema)) {
            this.setState(getInitialOneOfAnyOfState(nextProps.schema, nextProps.data));
        }
    }
    /**
     * Renders form items
     */
    renderFormItem() {
        if (isMapping(this.props.propertyName, this.props.componentMappingToPropertyNames)) {
            return this.renderMapping();
        }
        if (isSelect({ enum: this.props.schema.enum })) {
            return this.renderSelect();
        }
        switch (this.state.schema.type) {
            case "boolean":
                return this.renderCheckbox();
            case "number":
                return this.renderNumberField();
            case "string":
                return this.renderTextarea();
            case "array":
                return this.renderArray();
            case "children":
                return this.renderChildren();
            default:
                return this.renderSectionLink();
        }
    }
    renderChildren() {
        return (React.createElement(FormItemChildren, { required: false, index: this.props.index, label: this.props.schema.title || this.props.untitled, dataLocation: this.props.dataLocation, data: this.props.data, schema: this.props.schema, onChange: this.props.onChange, defaultChildOptions: this.props.schema.defaults || null, childOptions: this.props.childOptions, onUpdateActiveSection: this.props.onUpdateActiveSection }));
    }
    /**
     * Renders the array form item
     */
    renderArray() {
        return (React.createElement(FormItemArray, Object.assign({ schema: this.state.schema, schemaLocation: this.props.schemaLocation, untitled: this.props.untitled, onUpdateActiveSection: this.props.onUpdateActiveSection }, this.getFormItemCommonProps())));
    }
    /**
     * Renders the number field form item
     */
    renderNumberField() {
        return (React.createElement(FormItemNumberField, Object.assign({ min: this.props.schema.minimum, max: this.props.schema.maximum, step: this.props.schema.multipleOf }, this.getFormItemCommonProps())));
    }
    /**
     * Renders the checkbox form item
     */
    renderCheckbox() {
        return React.createElement(FormItemCheckbox, Object.assign({}, this.getFormItemCommonProps()));
    }
    /**
     * Renders the mapping form item which in turn
     * renders one of the special form items
     * assigned via the componentMappingToPropertyNames prop
     */
    renderMapping() {
        const name = formItemMapping(this.props.componentMappingToPropertyNames, this.props.propertyName);
        return (React.createElement(FormItemMapping, Object.assign({ name: name, options: this.props.schema.enum }, this.getFormItemCommonProps())));
    }
    /**
     * Renders a section link for properties
     * that are objects
     */
    renderSectionLink() {
        return (React.createElement(FormItemSectionLink, Object.assign({ onUpdateSection: this.handleSectionLinkClick, schemaLocation: this.props.schemaLocation }, this.getFormItemCommonProps())));
    }
    /**
     * Renders the textarea form item
     */
    renderTextarea() {
        if (this.props.attributeSettingsMappingToPropertyNames &&
            this.props.attributeSettingsMappingToPropertyNames.textarea) {
            const rowAttribute = formItemAttributeMapping(this.props.attributeSettingsMappingToPropertyNames
                .textarea, this.props.propertyName);
            return (React.createElement(FormItemTextarea, Object.assign({ rows: rowAttribute }, this.getFormItemCommonProps())));
        }
        return React.createElement(FormItemTextarea, Object.assign({}, this.getFormItemCommonProps()));
    }
    /**
     * Renders the select form item
     */
    renderSelect() {
        const options = this.props.schema.enum;
        if (!this.props.required && typeof options[0] !== "undefined") {
            options.unshift(void 0);
        }
        return React.createElement(FormItemSelect, Object.assign({ options: options }, this.getFormItemCommonProps()));
    }
    /**
     * Renders a select if the root level has a oneOf or anyOf
     */
    renderAnyOfOneOfSelect() {
        if (typeof this.state.oneOfAnyOf !== "undefined" &&
            this.props.schema[this.state.oneOfAnyOf.type]) {
            const options = getOneOfAnyOfSelectOptions(this.props.schema, this.state);
            return (React.createElement(FormOneOfAnyOf, { label: this.props.schema.title ||
                    this.props.schema.description ||
                    this.props.untitled, activeIndex: this.state.oneOfAnyOf.activeIndex, onUpdate: this.handleAnyOfOneOfClick }, options));
        }
        return null;
    }
    /**
     * Handles updating to another active section
     */
    handleUpdateSection(schemaLocation, dataLocation) {
        this.props.onUpdateActiveSection(schemaLocation, dataLocation, this.props.schema);
    }
    /**
     * Gets the common form item props
     */
    getFormItemCommonProps() {
        return {
            index: this.props.index,
            dataLocation: this.props.dataLocation,
            data: this.props.data,
            required: this.props.required,
            label: this.props.label,
            disabled: this.props.disabled,
            onChange: this.props.onChange,
            default: this.props.default,
        };
    }
}
FormControl.displayName = "FormControl";
export default FormControl;
