import * as React from "react";
import { get, uniqueId } from "lodash-es";
import { arrayMove, SortableContainer, SortableElement } from "react-sortable-hoc";
import { SortableListItem, sortingProps } from "./sorting";
import { generateExampleData } from "./form-section.utilities";
import { isRootLocation } from "./form.utilities";
import { getArrayLinks } from "./form-item.array.utilities";
import styles from "./form-item.array.style";
import manageJss from "@microsoft/fast-jss-manager-react";
import FormItemBase from "./form-item.base";
export var ItemConstraints;
(function (ItemConstraints) {
    ItemConstraints["minItems"] = "minItems";
    ItemConstraints["maxItems"] = "maxItems";
})(ItemConstraints || (ItemConstraints = {}));
export var ArrayAction;
(function (ArrayAction) {
    ArrayAction["add"] = "add";
    ArrayAction["remove"] = "remove";
})(ArrayAction || (ArrayAction = {}));
/**
 * Schema form component definition
 * @extends React.Component
 */
class FormItemArray extends FormItemBase {
    constructor() {
        super(...arguments);
        /**
         * Array section link click handler factory
         */
        this.arrayClickHandlerFactory = (item, index) => {
            return (e) => {
                e.preventDefault();
                const schemaLocation = isRootLocation(this.props.schemaLocation)
                    ? "items"
                    : `${this.props.schemaLocation}.items`;
                this.props.onUpdateActiveSection(schemaLocation, `${this.props.dataLocation}[${index}]`, this.props.schema);
            };
        };
        /**
         * Generates UI representing an item of an array
         */
        this.generateArrayLinkItem = (value, index) => {
            return (React.createElement(SortableListItem, { className: this.props.managedClasses.formItemArray_existingItemListItem, key: `item-${index}`, id: uniqueId(index.toString()) },
                React.createElement("a", { className: this.props.managedClasses.formItemArray_existingItemListItemLink, onClick: this.arrayClickHandlerFactory(value, index) }, value),
                this.renderDeleteArrayItemTrigger(index)));
        };
        /**
         * Handle user drag and drop interactions
         */
        this.handleSort = ({ oldIndex, newIndex }) => {
            this.props.onChange(this.props.dataLocation, arrayMove(this.props.data, oldIndex, newIndex));
        };
    }
    render() {
        return (React.createElement("div", { className: this.props.managedClasses.formItemArray },
            this.renderLabel(),
            this.renderExistingArrayItems()));
    }
    renderLabel() {
        const label = get(this.props.schema, "title") ||
            get(this.props.schema, "description") ||
            this.props.untitled;
        return (React.createElement("div", { className: this.props.managedClasses.formItemArray_control },
            React.createElement("label", { className: this.props.managedClasses.formItemArray_controlLabel }, label),
            this.renderAddArrayItemTrigger()));
    }
    /**
     * Render a button for adding an item to the array
     */
    renderAddArrayItemTrigger() {
        const maxItems = this.props.schemaLocation === ""
            ? get(this.props, "schema.maxItems", Infinity)
            : get(this.props, `schema.${this.props.schemaLocation}.maxItems`, Infinity);
        const items = Array.isArray(this.props.data) ? this.props.data.length : 0;
        if (maxItems > items) {
            return (React.createElement("button", { className: this.props.managedClasses.formItemArray_controlAddButton, "aria-label": "Select to add item", onClick: this.arrayItemClickHandlerFactory(this.props.dataLocation, ArrayAction.add) }));
        }
    }
    /**
     * Render a button for removing an item from the array
     */
    renderDeleteArrayItemTrigger(index) {
        const minItems = get(this.props, "schema.minItems", 0);
        if (!Array.isArray(this.props.data) ||
            (Array.isArray(this.props.data) && minItems < this.props.data.length)) {
            return (React.createElement("button", { className: this.props.managedClasses.formItemArray_existingItemRemoveButton, "aria-label": "Select to remove item", onClick: this.arrayItemClickHandlerFactory(this.props.dataLocation, ArrayAction.remove, index) }));
        }
    }
    /**
     * Array add/remove item click handler factory
     */
    arrayItemClickHandlerFactory(dataLocation, type, index) {
        return (e) => {
            e.preventDefault();
            type === ArrayAction.add
                ? this.handleAddArrayItem(dataLocation)
                : this.handleRemoveArrayItem(dataLocation, index);
        };
    }
    /**
     * Handles adding an array item
     */
    handleAddArrayItem(dataLocation) {
        if (typeof this.props.data === "undefined") {
            this.props.onChange(dataLocation, [
                generateExampleData(this.props.schema, "items"),
            ]);
        }
        else {
            this.props.onChange(dataLocation, generateExampleData(this.props.schema, "items"), true);
        }
    }
    /**
     * Handles removing an array item
     */
    handleRemoveArrayItem(dataLocation, index) {
        this.props.onChange(dataLocation, void 0, true, index);
    }
    /**
     * Generates UI for all items in an array
     */
    generateArrayLinkItems() {
        return getArrayLinks(this.props.data).map((value, index) => {
            const options = {
                key: `item-${index}`,
                index,
                value,
            };
            return React.createElement(SortableElement(this.generateArrayLinkItem.bind(this, value, index)), options);
        });
    }
    /**
     * Generates the links to an array section to be activated
     */
    renderExistingArrayItems() {
        const arraySections = getArrayLinks(this.props.data);
        const props = Object.assign({}, sortingProps, {
            onSortEnd: this.handleSort,
            helperClass: this.props.managedClasses
                .formItemArray_existingItemListItem__sorting,
        });
        if (arraySections.length > 0) {
            return React.createElement(SortableContainer(() => {
                return (React.createElement("ul", { className: this.props.managedClasses.formItemArray_existingItemList }, this.generateArrayLinkItems()));
            }), props);
        }
    }
}
FormItemArray.displayName = "FormItemArray";
export { FormItemArray };
export default manageJss(styles)(FormItemArray);
