import { generateExampleData } from "./form-section.utilities";
import * as React from "react";
import { canUseDOM } from "exenv-es6";
import { arrayMove, SortableContainer, SortableElement } from "react-sortable-hoc";
import { get } from "lodash-es";
import { getChildOptionBySchemaId } from "@microsoft/fast-data-utilities-react";
import { SortableListItem, sortingProps } from "./sorting";
import { cloneDeep, uniqueId } from "lodash-es";
import { reactChildrenStringSchema } from "./form-item.children.text";
import styles from "./form-item.children.style";
import { KeyCodes } from "@microsoft/fast-web-utilities";
import manageJss from "@microsoft/fast-jss-manager-react";
import FormItemBase from "./form-item.base";
export var Action;
(function (Action) {
    Action["add"] = "add";
    Action["edit"] = "edit";
    Action["delete"] = "delete";
})(Action || (Action = {}));
/**
 * Schema form component definition
 * @extends React.Component
 */
/* tslint:disable-next-line */
class FormItemChildren extends FormItemBase {
    constructor(props) {
        super(props);
        /**
         * Renders an existing child item
         */
        this.renderExistingChild = (item, index) => {
            // The "react-sortable-hoc" library has an issue where a wrapping element
            // must be supplied in order to accurately show drag movement
            // see: https://github.com/clauderic/react-sortable-hoc/issues/305
            return (React.createElement(SortableListItem, { className: this.props.managedClasses.formItemChildren_existingChildrenItem, key: `item-${index}`, id: uniqueId(index ? index.toString() : "") },
                React.createElement("a", { "aria-label": "Select to edit", className: this.props.managedClasses
                        .formItemChildren_existingChildrenItemLink, onClick: this.clickEditComponentFactory(item, index) },
                    React.createElement("span", { className: this.props.managedClasses
                            .formItemChildren_existingChildrenItemName }, this.generateChildOptionText(item)),
                    this.renderExistingChildCaption(item)),
                this.renderExistingChildDelete(index)));
        };
        /**
         * Keydown handler for the child option filter
         */
        this.handleChildrenListInputKeydown = (e) => {
            switch (e.keyCode) {
                case KeyCodes.enter:
                    e.preventDefault();
                case KeyCodes.tab:
                    if (this.state.childrenSearchTerm !== "") {
                        this.onAddComponent(this.state.filteredChildOptions[this.state.indexOfSelectedFilteredChildOption]);
                    }
                    this.setState({
                        hideChildrenList: true,
                    });
                    break;
                case KeyCodes.arrowUp:
                    this.selectPreviousFilteredChildOption();
                    break;
                case KeyCodes.arrowDown:
                    this.selectNextFilteredChildOption();
                    break;
            }
        };
        /**
         * Click handler for the children list trigger
         */
        this.handleChildrenListTriggerClick = (e) => {
            this.handleToggleChildrenListVisibility(e);
            if (this.filteredChildrenInput.current instanceof HTMLElement) {
                this.filteredChildrenInput.current.focus();
            }
        };
        /**
         * Click handler for toggling the visibility of the children list
         */
        this.handleToggleChildrenListVisibility = (e) => {
            this.setState({
                hideChildrenList: !this.state.hideChildrenList,
            });
            if (this.filteredChildrenList.current instanceof HTMLElement) {
                this.filteredChildrenList.current.scrollTop = 0;
            }
        };
        /**
         * Click handler for the window
         */
        this.handleWindowClick = (e) => {
            if (this.isTargetingChildrenList(e)) {
                this.setState({ hideChildrenList: true });
            }
        };
        /**
         * Callback to call when children sorting has occured
         */
        this.handleSort = ({ oldIndex, newIndex }) => {
            const childrenData = cloneDeep(this.props.data);
            if (Boolean(childrenData)) {
                this.props.onChange(this.props.dataLocation, arrayMove(childrenData, oldIndex, newIndex));
            }
        };
        /**
         * Change handler for editing the child option filter
         */
        this.handleChildOptionFilterInputChange = (e) => {
            const filteredChildOptions = this.childOptions.filter((option) => {
                return option.name.toLowerCase().includes(e.target.value.toLowerCase());
            });
            this.setState({
                childrenSearchTerm: e.target.value,
                indexOfSelectedFilteredChildOption: 0,
                filteredChildOptions,
                hideChildrenList: e.target.value === "",
            });
        };
        /**
         * Click factory for adding a child item
         */
        this.clickAddComponentFactory = (component) => {
            return (e) => {
                this.onAddComponent(component);
            };
        };
        /**
         * Click factory for editing a child item
         */
        this.clickEditComponentFactory = (component, index) => {
            return (e) => {
                e.preventDefault();
                this.onEditComponent(component, index);
            };
        };
        /**
         * Click factory for removing a child item
         */
        this.clickDeleteComponentFactory = (index) => {
            return (e) => {
                e.preventDefault();
                this.onDeleteComponent(index);
            };
        };
        this.filteredChildrenList = React.createRef();
        this.filteredChildrenListTrigger = React.createRef();
        this.filteredChildrenInput = React.createRef();
        this.selectedChildOption = React.createRef();
        const defaultOptions = [];
        if (Array.isArray(this.props.defaultChildOptions) &&
            this.props.defaultChildOptions.includes("text")) {
            defaultOptions.push({
                name: "Text",
                component: null,
                schema: reactChildrenStringSchema,
            });
        }
        this.childOptions = defaultOptions.concat(this.props.childOptions);
        this.state = {
            childrenSearchTerm: "",
            indexOfSelectedFilteredChildOption: 0,
            filteredChildOptions: this.childOptions,
            hideChildrenList: true,
        };
    }
    render() {
        // Convert to search component when #3006 has been completed
        return (React.createElement("div", { className: this.props.managedClasses.formItemChildren },
            React.createElement("div", { className: this.props.managedClasses.formItemChildren_control },
                React.createElement("label", { className: this.props.managedClasses.formItemChildren_controlLabel, id: this.getLabelId() }, this.props.label)),
            this.renderExistingChildren(),
            this.renderAddChild()));
    }
    componentDidMount() {
        if (canUseDOM()) {
            document.addEventListener("click", this.handleWindowClick);
        }
    }
    componentWillUnmount() {
        if (canUseDOM()) {
            document.removeEventListener("click", this.handleWindowClick);
        }
    }
    /**
     * Renders the input, button and listbox for
     * adding a child
     */
    renderAddChild() {
        return (React.createElement("div", null,
            React.createElement("div", { className: this.props.managedClasses.formItemChildren_childrenListControl },
                React.createElement("input", { className: this.props.managedClasses.formItemChildren_childrenListInput, type: "text", "aria-autocomplete": "list", "aria-controls": this.getFilteredChildrenInputId(), "aria-labelledby": this.getLabelId(), value: this.state.childrenSearchTerm, placeholder: "Add", onChange: this.handleChildOptionFilterInputChange, onKeyDown: this.handleChildrenListInputKeydown, ref: this.filteredChildrenInput }),
                React.createElement("button", { className: this.props.managedClasses.formItemChildren_childrenListTrigger, tabIndex: -1, "aria-label": "Show children options", onClick: this.handleChildrenListTriggerClick, ref: this.filteredChildrenListTrigger })),
            React.createElement("ul", { id: this.getFilteredChildrenInputId(), "aria-labelledby": this.getLabelId(), "aria-hidden": this.state.hideChildrenList, className: this.props.managedClasses.formItemChildren_childrenList, role: "listbox", ref: this.filteredChildrenList }, this.renderFilteredChildOptions())));
    }
    /**
     * Renders the optional children
     */
    renderFilteredChildOptions() {
        return this.state.filteredChildOptions.map((option, index) => {
            const selected = this.state.filteredChildOptions[this.state.indexOfSelectedFilteredChildOption] === option;
            return (React.createElement("li", { className: this.props.managedClasses.formItemChildren_childrenListItem, key: uniqueId(), role: "option", "aria-selected": selected, onClick: this.clickAddComponentFactory(option), ref: selected ? this.selectedChildOption : null },
                React.createElement("span", null, option.name)));
        });
    }
    /**
     * Renders a caption for an existing child item
     */
    renderExistingChildCaption(instance) {
        if (instance &&
            instance.props &&
            instance.props.text) {
            return (React.createElement(React.Fragment, null,
                React.createElement("br", null),
                React.createElement("i", { className: this.props.managedClasses
                        .formItemChildren_existingChildrenItemContent }, instance.props.text)));
        }
        return null;
    }
    /**
     * Renders the delete button for an existing child
     */
    renderExistingChildDelete(index) {
        return (React.createElement("button", { "aria-label": "Select to remove", className: this.props.managedClasses.formItemChildren_deleteButton, onClick: this.clickDeleteComponentFactory(index) }));
    }
    /**
     * Generate the text to use as display for a child option
     */
    generateChildOptionText(item) {
        const childOption = getChildOptionBySchemaId(item.id, this.childOptions);
        if (typeof childOption === "object" && childOption !== null) {
            return childOption.name;
        }
        else {
            return this.getChildOptionTextString(item);
        }
    }
    /**
     * Generate all items for the list of existing children
     */
    renderExistingChildItems() {
        const currentChildren = this.props.data;
        const currentChildrenArray = Array.isArray(currentChildren)
            ? currentChildren
            : [currentChildren];
        if (currentChildren) {
            return currentChildrenArray.map((item, index) => {
                const options = {
                    key: `item-${index}`,
                    index,
                    value: typeof item === "object"
                        ? this.generateChildOptionText(item)
                        : item,
                };
                return React.createElement(SortableElement(this.renderExistingChild.bind(this, item, currentChildrenArray.length > 1 ? index : undefined)), options);
            });
        }
        return null;
    }
    /**
     * Render the list of existing children for a component
     */
    renderExistingChildren() {
        const props = Object.assign({}, sortingProps, {
            onSortEnd: this.handleSort,
            helperClass: this.props.managedClasses
                .formItemChildren_existingChildrenItem__sorting,
        });
        const childItems = this.renderExistingChildItems();
        if (childItems) {
            return React.createElement(SortableContainer(() => {
                return (React.createElement("ul", { className: this.props.managedClasses
                        .formItemChildren_existingChildren }, childItems));
            }), props);
        }
        return null;
    }
    /**
     * Click handler for editing a component
     */
    onEditComponent(component, index) {
        let childSchema;
        if (typeof component === "string") {
            childSchema = reactChildrenStringSchema;
        }
        else if (typeof component === "object" &&
            typeof component.props === "object") {
            this.childOptions.forEach((childOption) => {
                if (childOption.schema.id === component.id) {
                    childSchema = childOption.schema;
                }
            });
        }
        const dataLocation = this.getDataLocation(component, index);
        this.props.onUpdateActiveSection("", dataLocation, childSchema);
    }
    /**
     * Click handler for deleting a component
     */
    onDeleteComponent(index) {
        this.props.onChange(this.props.dataLocation, undefined, typeof index === "number", index, true);
    }
    /**
     * Click event for adding a component
     */
    onAddComponent(item) {
        const currentChildren = this.getCurrentChildren(this.props.data);
        if (typeof item === "object" && item !== null) {
            this.props.onChange(this.props.dataLocation, this.getChildComponents(currentChildren, item), undefined, undefined, true);
        }
        else if (typeof item === "string") {
            this.props.onChange(this.props.dataLocation, this.getChildStrings(currentChildren, item), undefined, undefined, true);
        }
        this.setState({
            hideChildrenList: true,
            childrenSearchTerm: "",
        });
    }
    /**
     * Updates the filtered child options to select the next child
     */
    selectNextFilteredChildOption() {
        if (this.state.filteredChildOptions.length > 1) {
            if (this.state.indexOfSelectedFilteredChildOption ===
                this.state.filteredChildOptions.length - 1) {
                this.setState({
                    indexOfSelectedFilteredChildOption: 0,
                });
                this.filteredChildrenList.current.scrollTop = 0;
            }
            else {
                this.setState({
                    indexOfSelectedFilteredChildOption: this.state.indexOfSelectedFilteredChildOption + 1,
                });
                this.filteredChildrenList.current.scrollTop = this.selectedChildOption
                    .current.nextSibling.offsetTop;
            }
        }
    }
    /**
     * Updates the filtered child options to select the previous child
     */
    selectPreviousFilteredChildOption() {
        if (this.state.filteredChildOptions.length > 1) {
            if (this.state.indexOfSelectedFilteredChildOption === 0) {
                this.setState({
                    indexOfSelectedFilteredChildOption: this.state.filteredChildOptions.length - 1,
                });
                this.filteredChildrenList.current.scrollTop = this.filteredChildrenList.current.scrollHeight;
            }
            else {
                this.setState({
                    indexOfSelectedFilteredChildOption: this.state.indexOfSelectedFilteredChildOption - 1,
                });
                this.filteredChildrenList.current.scrollTop = this.selectedChildOption
                    .current.previousSibling.offsetTop;
            }
        }
    }
    isTargetingChildrenList(e) {
        return (e.target instanceof Element &&
            get(this.filteredChildrenList, "current") &&
            get(this.filteredChildrenListTrigger, "current") &&
            !this.filteredChildrenList.current.contains(e.target) &&
            !this.filteredChildrenListTrigger.current.contains(e.target) &&
            this.filteredChildrenListTrigger.current !== e.target);
    }
    getLabelId() {
        return `${this.props.dataLocation}-label`;
    }
    getFilteredChildrenInputId() {
        return `${this.props.dataLocation}-input`;
    }
    getChildComponents(currentChildren, item) {
        const components = [].concat(currentChildren);
        const isString = item.schema === reactChildrenStringSchema;
        components.push(this.getChildComponent(item, isString));
        return components;
    }
    getChildStrings(currentChildren, item) {
        const components = currentChildren;
        if (components.length > 0) {
            components.push(item);
        }
        return components.length > 0 ? components : item;
    }
    getCurrentChildren(currentChildren) {
        return Array.isArray(currentChildren)
            ? currentChildren
            : typeof currentChildren !== "undefined" && currentChildren !== null
                ? [currentChildren]
                : [];
    }
    getChildComponent(item, isString) {
        if (!!isString) {
            return generateExampleData(item.schema, "");
        }
        return {
            id: item.schema.id,
            props: generateExampleData(item.schema, ""),
        };
    }
    getDataLocation(component, index) {
        const propLocation = typeof component === "string" ? "" : ".props";
        if (typeof index === "number") {
            return `${this.props.dataLocation}[${index}]${propLocation}`;
        }
        return `${this.props.dataLocation}${propLocation}`;
    }
    getChildOptionTextString(item) {
        const textString = typeof item.props === "string" ? item.props : item;
        return textString ? textString : "Untitled";
    }
}
FormItemChildren.displayName = "FormItemChildren";
export default manageJss(styles)(FormItemChildren);
