import * as React from "react";
import { uniqueId } from "lodash-es";
/**
 * Schema form component definition
 * @extends React.Component
 */
class FormItemFileUpload extends React.Component {
    constructor(props) {
        super(props);
        /**
         * File reader to handle reading / parsing of file data
         */
        this.reader = new FileReader();
        /**
         * Callback to call when the reader loads file data
         */
        this.handleReaderLoad = () => {
            this.setState({ processing: false });
            this.props.onChange(this.props.dataLocation, this.reader.result);
        };
        /**
         * Event handler that effectively cancels the event
         */
        this.cancelEvent = (e) => {
            e.preventDefault();
        };
        /**
         * Event handler for drag-area dragover event
         */
        this.dragOver = (e) => {
            this.cancelEvent(e);
            if (!this.state.dragging) {
                this.setState({ dragging: true });
            }
        };
        /**
         * Event handler for drag-area drag-leave event
         */
        this.dragLeave = (e) => {
            this.cancelEvent(e);
            this.setState({ dragging: false });
        };
        /**
         * Callback for drop event
         */
        this.onDrop = (e) => {
            this.cancelEvent(e);
            this.setState({ dragging: false });
            this.updateWithFile(e.dataTransfer.files[0]);
        };
        /**
         * Callback for input change
         */
        this.handleInputOnChange = (e) => {
            this.updateWithFile(e.target.files[0]);
        };
        this.fileId = uniqueId();
        this.state = {
            dragging: false,
            processing: false,
        };
    }
    /**
     * Render the component
     */
    render() {
        return (React.createElement("div", null,
            React.createElement("label", { htmlFor: this.fileId }, this.props.label),
            React.createElement("div", { onDragEnter: this.cancelEvent, onDragOver: this.dragOver, onDragLeave: this.dragLeave, onDrop: this.onDrop }, this.state.processing
                ? this.generateProcessingUI()
                : this.generateStaticUI())));
    }
    /**
     * React lifecycle hook
     */
    componentDidMount() {
        this.reader.addEventListener("load", this.handleReaderLoad);
    }
    /**
     * React lifecycle hook
     */
    componentWillUnMount() {
        this.reader.removeEventListener("load", this.handleReaderLoad);
    }
    /**
     * Updates the component with new file data
     */
    updateWithFile(file) {
        // We should exit if file is not actually a File or the type is not an image
        if (!(file instanceof File) || !file.type.includes("image")) {
            return;
        }
        this.setState({ processing: true });
        this.reader.readAsDataURL(file);
    }
    generateStaticUI() {
        return [
            typeof this.props.data === "string" ? (React.createElement("img", { key: "thumbnail", src: this.props.data, alt: "Image upload thumbnail" })) : null,
            React.createElement("input", { key: "input", type: "file", id: this.fileId, onChange: this.handleInputOnChange }),
            React.createElement("p", { key: "info" },
                "Drag your asset here or",
                " ",
                React.createElement("label", { htmlFor: this.fileId }, "browse your files.")),
        ];
    }
    generateProcessingUI() {
        return React.createElement("p", null, "Uploading...");
    }
}
FormItemFileUpload.displayName = "FormItemFileUpload";
export default FormItemFileUpload;
