import styles from "./form-section.style";
import * as React from "react";
import FormCategory from "./form-category";
import { get, uniqueId } from "lodash-es";
import { checkCategoryConfigPropertyCount, checkIsDifferentSchema, checkIsObject, generateExampleData, getCategoryParams, getData, getInitialOneOfAnyOfState, getIsNotRequired, getIsRequired, getLabel, getOneOfAnyOfSelectOptions, } from "./form-section.utilities";
import manageJss from "@microsoft/fast-jss-manager-react";
import FormControl from "./form-control";
import FormOneOfAnyOf from "./form-one-of-any-of";
/**
 * Schema form component definition
 * @extends React.Component
 */
class FormSection extends React.Component {
    constructor(props) {
        super(props);
        /**
         * Handles updating the schema to another active oneOf/anyOf schema
         */
        this.handleAnyOfOneOfClick = (activeIndex) => {
            const updatedData = generateExampleData(this.props.schema[this.state.oneOfAnyOf.type][activeIndex], "");
            this.props.onChange(this.props.dataLocation, updatedData);
            this.setState({
                schema: this.props.schema[this.state.oneOfAnyOf.type][activeIndex],
                oneOfAnyOf: {
                    type: this.state.oneOfAnyOf.type,
                    activeIndex,
                },
            });
        };
        this.state = getInitialOneOfAnyOfState(this.props.schema, this.props.data);
    }
    render() {
        return React.createElement("div", null, this.renderFormSection());
    }
    /**
     * React lifecycle hook
     */
    componentWillUpdate(nextProps) {
        if (checkIsDifferentSchema(this.props.schema, nextProps.schema)) {
            this.setState(getInitialOneOfAnyOfState(nextProps.schema, nextProps.data));
        }
    }
    /**
     * Renders the form
     */
    renderForm(dataLocation, schema) {
        // check to see if this is a root level object
        // if so, use it to generate the form and do not generate a link
        if (dataLocation === "") {
            return this.generateFormObject(schema, schema.required || undefined, schema.not ? schema.not.required : undefined);
        }
    }
    /**
     * Generates form elements based on field type
     */
    renderFormControl(schema, index, propertyName, schemaLocation, dataLocation, required, label) {
        if (schema.type === "object" && propertyName === "") {
            return this.renderForm(propertyName, schema);
        }
        return (React.createElement(FormControl, { key: [this.props.dataLocation, propertyName].join("."), index: index, untitled: this.props.untitled, required: required, default: schema.default || void 0, label: getLabel(label, this.state.schema.title), data: getData(propertyName, this.props.data), dataLocation: dataLocation, schemaLocation: schemaLocation, childOptions: this.props.childOptions, propertyName: propertyName, schema: schema, onChange: this.props.onChange, onUpdateActiveSection: this.props.onUpdateActiveSection, attributeSettingsMappingToPropertyNames: this.props.attributeSettingsMappingToPropertyNames, componentMappingToPropertyNames: this.props.componentMappingToPropertyNames }));
    }
    /**
     * Renders form items into categories
     */
    renderCategories(formItemParameters) {
        const categoryParams = getCategoryParams(formItemParameters, this.props.orderByPropertyNames);
        return categoryParams.map((categoryParam, index) => {
            return (React.createElement(FormCategory, { key: index, expandable: categoryParam.expandable, title: categoryParam.title }, this.renderCategoryItems(categoryParam.items)));
        });
    }
    /**
     * Renders category items
     */
    renderCategoryItems(items) {
        return items.map((item) => {
            return this.renderFormControl(item.params.schema, item.params.index, item.params.propertyName, item.params.schemaLocation, item.params.dataLocation, item.params.isRequired, item.params.title);
        });
    }
    getFormItemsAndConfigurationOptions(schema, required, not) {
        const formItems = {
            items: [],
            parameters: [],
        };
        const propertyKeys = [];
        if (schema.properties) {
            propertyKeys.push("properties");
        }
        if (schema.reactProperties) {
            propertyKeys.push("reactProperties");
        }
        propertyKeys.forEach((propertyKey) => {
            Object.keys(schema[propertyKey]).forEach((propertyName, index) => {
                const isRequired = getIsRequired(propertyName, required);
                const isNotRequired = getIsNotRequired(propertyName, not);
                const schemaLocation = this.getSchemaLocation() === ""
                    ? [propertyKey, propertyName].join(".")
                    : [
                        this.getSchemaLocation(),
                        propertyKey,
                        propertyName,
                    ].join(".");
                const dataLocation = this.props.dataLocation !== ""
                    ? [this.props.dataLocation, propertyName].join(".")
                    : propertyName;
                if (!isNotRequired) {
                    const params = {
                        property: schema[propertyKey][propertyName],
                        index,
                        schemaLocation,
                        dataLocation,
                        propertyName,
                        schema: get(this.state.schema, `${propertyKey}.${propertyName}`),
                        isRequired,
                        title: schema[propertyKey][propertyName].title ||
                            this.props.untitled,
                    };
                    formItems.items.push(this.renderFormControl(params.property, params.index, params.propertyName, params.schemaLocation, params.dataLocation, params.isRequired, params.title));
                    if (params.property.type !== "object" &&
                        typeof params.property[propertyKey] === "undefined") {
                        formItems.parameters.push(params);
                    }
                }
            });
        });
        return formItems;
    }
    getFormObjectItemsOrConfigCategories(formItems) {
        if (this.props.orderByPropertyNames) {
            if (checkCategoryConfigPropertyCount(formItems, this.props.orderByPropertyNames)) {
                return formItems.items;
            }
            return this.renderCategories(formItems.parameters);
        }
        return formItems.items;
    }
    /**
     * Pushes a list of elements found inside an object to an array
     */
    generateFormObject(schema, required, not) {
        let formItems;
        if (checkIsObject(schema, this.state.schema)) {
            // assign items to form elements
            formItems = this.getFormItemsAndConfigurationOptions(schema, required, not);
            return this.getFormObjectItemsOrConfigCategories(formItems);
        }
    }
    /**
     * Renders a select if the root level has a oneOf or anyOf
     */
    renderAnyOfOneOfSelect() {
        if (typeof this.state.oneOfAnyOf !== "undefined" &&
            this.props.schema[this.state.oneOfAnyOf.type]) {
            const id = uniqueId();
            const options = getOneOfAnyOfSelectOptions(this.props.schema, this.state);
            return (React.createElement(FormOneOfAnyOf, { label: "Configuration", activeIndex: this.state.oneOfAnyOf.activeIndex, onUpdate: this.handleAnyOfOneOfClick }, options));
        }
        return null;
    }
    renderFormSection() {
        return (React.createElement("div", null,
            React.createElement("div", null,
                this.renderAnyOfOneOfSelect(),
                this.renderFormControl(this.state.schema, 0, "", this.getSchemaLocation(), this.props.dataLocation, true, ""))));
    }
    getSchemaLocation() {
        if (this.state.oneOfAnyOf) {
            const separator = this.props.schemaLocation === "" ? "" : ".";
            return `${this.props.schemaLocation}${separator}${this.state.oneOfAnyOf.type}[${this.state.oneOfAnyOf.activeIndex}]`;
        }
        else {
            return this.props.schemaLocation;
        }
    }
}
FormSection.displayName = "FormSection";
export default manageJss(styles)(FormSection);
