import * as React from "react";
import { cloneDeep, get, mergeWith, set, unset } from "lodash-es";
import { getExample } from "@microsoft/fast-permutator";
import tv4 from "tv4";
import { isRootLocation } from "./form.utilities";
import { oneOfAnyOfType, } from "./form-section.props";
export function getInitialOneOfAnyOfState(schema, data) {
    let oneOfAnyOf;
    let oneOfAnyOfState;
    let activeIndex;
    let updatedSchema = schema;
    if (schema.oneOf || schema.anyOf) {
        oneOfAnyOf = schema.oneOf ? oneOfAnyOfType.oneOf : oneOfAnyOfType.anyOf;
        activeIndex = getOneOfAnyOfActiveIndex(oneOfAnyOf, schema, data);
        updatedSchema = schema[oneOfAnyOf][activeIndex];
        oneOfAnyOfState = {
            type: oneOfAnyOf,
            activeIndex,
        };
    }
    return {
        schema: updatedSchema,
        oneOfAnyOf: oneOfAnyOfState,
    };
}
/**
 * Validate a schema against a set of data
 */
export function validateSchema(schema, data) {
    return tv4 ? tv4.validate(data, schema) : false;
}
/**
 * Determine if an item is required
 */
export function getIsRequired(item, required) {
    let isRequired = false;
    if (Array.isArray(required)) {
        required.forEach((requiredItem) => {
            if (requiredItem === item) {
                isRequired = true;
            }
        });
    }
    return isRequired;
}
/**
 * Determine if an item is not required
 */
export function getIsNotRequired(item, not) {
    let notRequired = false;
    if (Array.isArray(not)) {
        not.forEach((notRequiredItem) => {
            if (notRequiredItem === item) {
                notRequired = true;
            }
        });
    }
    return notRequired;
}
/**
 * Gets the options for a oneOf/anyOf select
 */
export function getOneOfAnyOfSelectOptions(schema, state) {
    return schema[state.oneOfAnyOf.type].map((option, index) => {
        return (React.createElement("option", { key: index, value: index }, get(option, "title") || get(option, "description") || "No title"));
    });
}
function removeUndefinedKeys(data) {
    const clonedData = cloneDeep(data);
    if (clonedData !== undefined) {
        Object.keys(clonedData).forEach((key) => {
            if (typeof clonedData[key] === "undefined") {
                // if this is a child we may be getting undefined default props, remove these
                delete clonedData[key];
            }
        });
    }
    return clonedData;
}
function checkSchemaTypeIsArray(schema, type) {
    return schema && Array.isArray(schema[type]);
}
/**
 * Find out what the active index should be based on the data
 */
export function getOneOfAnyOfActiveIndex(type, schema, data) {
    let activeIndex = 0;
    if (checkSchemaTypeIsArray(schema, type)) {
        const newData = removeUndefinedKeys(data);
        schema[type].forEach((oneOfAnyOfItem, index) => {
            if (validateSchema(oneOfAnyOfItem, newData)) {
                activeIndex = index;
                return;
            }
        });
    }
    return activeIndex;
}
/**
 * Resolves generated example data with any matching data in the cache
 */
export function resolveExampleDataWithCachedData(schema, cachedData) {
    const exampleData = generateExampleData(schema, "");
    const curatedCachedData = cloneDeep(cachedData);
    // removes any cached items which do not match and item in
    // the example data and are not included in the schema properties
    Object.keys(curatedCachedData).forEach((item) => {
        if (typeof exampleData[item] === "undefined" &&
            (schema.properties && !schema.properties[item])) {
            unset(curatedCachedData, item);
        }
    });
    // look through the data cache, find any matching properties and merge them together
    return mergeWith(exampleData, curatedCachedData, cachedDataResolver.bind(schema));
}
/**
 * Resolves cached data into example data if the schema still validates with the new data
 */
function cachedDataResolver(objValue, srcValue, key, object) {
    if (typeof srcValue !== "undefined" &&
        typeof objValue !== "undefined" &&
        typeof srcValue === typeof objValue) {
        const newObj = cloneDeep(object);
        set(newObj, key, srcValue);
        if (!validateSchema(this, newObj)) {
            return objValue;
        }
        return srcValue;
    }
    else {
        return void 0;
    }
}
/**
 * Normalizes a location for getting and setting values
 */
export function getNormalizedLocation(location, property, schema) {
    let normalizedProperty = "";
    let normalizedLocation = location === "" || property === "" ? location : `${location}.`;
    normalizedProperty = property.split(".").join(".properties.");
    normalizedLocation =
        typeof property !== "undefined"
            ? `${normalizedLocation}${normalizedProperty}`
            : normalizedLocation;
    return normalizedLocation;
}
function getArrayExample(schemaSection) {
    const example = getExample(schemaSection.items);
    if (schemaSection.minItems) {
        return new Array(schemaSection.length - 1).fill(example);
    }
    return [example];
}
function checkIsObjectAndSetType(schemaSection) {
    if (schemaSection.properties && schemaSection.type !== "object") {
        return "object";
    }
    return schemaSection.type;
}
/**
 * Generates example data for a newly added optional schema item
 */
export function generateExampleData(schema, propertyLocation) {
    let schemaSection = propertyLocation === "" ? schema : get(schema, propertyLocation);
    if (schemaSection.oneOf) {
        schemaSection = schemaSection.oneOf[0];
    }
    if (schemaSection.anyOf) {
        schemaSection = schemaSection.anyOf[0];
    }
    schemaSection.type = checkIsObjectAndSetType(schemaSection);
    if (schemaSection.items) {
        return getArrayExample(schemaSection);
    }
    return getExample(schemaSection);
}
/**
 * Get the array location
 */
export function getArraySchemaLocation(schemaLocation, propertyName, schema, oneOfAnyOf) {
    let arraySchemaLocation = "";
    if (oneOfAnyOf) {
        arraySchemaLocation += `${oneOfAnyOf.type}[${oneOfAnyOf.activeIndex}]`;
    }
    if (propertyName !== "") {
        arraySchemaLocation += `${arraySchemaLocation === "" ? "" : "."}properties.${propertyName}`;
    }
    return arraySchemaLocation;
}
/**
 * Assigns a schema form item mapping based on property name
 */
export function formItemMapping(config, propertyName) {
    let itemLayout = null;
    Object.keys(config).forEach((layout) => {
        for (const property of config[layout]) {
            if (property === propertyName) {
                itemLayout = layout;
            }
        }
    });
    return itemLayout;
}
/**
 * Assigns an attribute value based on property names
 */
export function formItemAttributeMapping(config, propertyName) {
    let itemAttributeValue = null;
    Object.keys(config).forEach((attributeName) => {
        for (const attributeConfig of config[attributeName]) {
            for (const property of attributeConfig.propertyNames) {
                if (property === propertyName) {
                    itemAttributeValue = attributeConfig.value;
                }
            }
        }
    });
    return itemAttributeValue;
}
export function checkHasOneOfAnyOf(oneOf, anyOf) {
    return oneOf || anyOf;
}
export function checkIsDifferentSchema(currentSchema, nextSchema) {
    return nextSchema !== currentSchema;
}
export function checkIsDifferentData(currentData, nextData) {
    return currentData !== nextData;
}
export function getOneOfAnyOfState(oneOfAnyOf, nextProps) {
    const oneOfAnyOfState = {};
    oneOfAnyOfState.type = nextProps.schema.oneOf
        ? oneOfAnyOfType.oneOf
        : oneOfAnyOfType.anyOf;
    oneOfAnyOfState.activeIndex = getOneOfAnyOfActiveIndex(oneOfAnyOfState.type, nextProps.schema, nextProps.data);
    return oneOfAnyOfState;
}
export function getDataLocationRelativeToRoot(location, dataLocation) {
    return isRootLocation(dataLocation) || isRootLocation(location)
        ? `${dataLocation}${location}`
        : `${dataLocation}.${location}`;
}
export function getData(location, data) {
    return isRootLocation(location) ? data : get(data, location);
}
export function isSelect(property) {
    return typeof property.enum !== "undefined" && property.enum.length > 0;
}
/**
 * Organizes the categories and items by weight
 */
export function getWeightedCategoriesAndItems(categoryParams) {
    categoryParams.sort(function (a, b) {
        return b.weight - a.weight;
    });
    for (const categoryParam of categoryParams) {
        categoryParam.items.sort(function (a, b) {
            return b.weight - a.weight;
        });
    }
    return categoryParams;
}
export function checkIsObject(property, schema) {
    return (property.properties || property.type === "object") && property === schema;
}
export function findAssignedParamsByCategoryProperties(config) {
    for (const propertyName of config.categoryProperties) {
        if (propertyName === config.formItemParameter.propertyName) {
            return {
                category: config.category.title,
                expandable: config.category.expandable,
                categoryWeight: config.category.weight,
                itemWeight: config.categoryProperty.weight || config.assignedItemWeight,
            };
        }
    }
}
export function getCategoryIndex(assignedCategoryParams, categoryParams) {
    for (let i = 0, categoryParamsLength = categoryParams.length; i < categoryParamsLength; i++) {
        if (assignedCategoryParams.category === categoryParams[i].title) {
            return i;
        }
    }
}
export function checkCategoryConfigPropertyCount(formItems, orderByPropertyNames) {
    return (typeof orderByPropertyNames.showCategoriesAtPropertyCount === "number" &&
        orderByPropertyNames.showCategoriesAtPropertyCount >= formItems.items.length);
}
export function findOrderedByPropertyNames(category, formItemParameter, assignedItemWeight) {
    for (const categoryProperty of category.properties) {
        const categoryProperties = Array.isArray(categoryProperty.propertyName)
            ? categoryProperty.propertyName
            : [categoryProperty.propertyName];
        const assignedParamsByCategoryProperties = findAssignedParamsByCategoryProperties({
            categoryProperties,
            formItemParameter,
            category,
            categoryProperty,
            assignedItemWeight,
        });
        if (Boolean(assignedParamsByCategoryProperties)) {
            return assignedParamsByCategoryProperties;
        }
    }
}
function getAssignedCategoryParams(formItemParameter, assignedItemWeight, orderByPropertyNames) {
    for (const category of orderByPropertyNames.categories) {
        const formItemOrderedByPropertyNames = findOrderedByPropertyNames(category, formItemParameter, assignedItemWeight);
        if (Boolean(formItemOrderedByPropertyNames)) {
            return formItemOrderedByPropertyNames;
        }
    }
    return {
        category: "Default",
        categoryWeight: orderByPropertyNames.defaultCategoryWeight || 0,
        itemWeight: 0,
    };
}
export function getCategoryParams(formItemParameters, orderByPropertyNames) {
    const categoryParams = [];
    for (const formItemParameter of formItemParameters) {
        const assignedCategoryParams = getAssignedCategoryParams(formItemParameter, 0, orderByPropertyNames);
        const categoryIndex = getCategoryIndex(assignedCategoryParams, categoryParams);
        if (typeof categoryIndex === "number") {
            categoryParams[categoryIndex].items.push({
                weight: assignedCategoryParams.itemWeight,
                params: formItemParameter,
            });
        }
        else {
            categoryParams.push({
                title: assignedCategoryParams.category,
                weight: assignedCategoryParams.categoryWeight,
                expandable: assignedCategoryParams.expandable,
                items: [
                    {
                        weight: assignedCategoryParams.itemWeight,
                        params: formItemParameter,
                    },
                ],
            });
        }
    }
    return getWeightedCategoriesAndItems(categoryParams);
}
export function handleToggleClick(value, id, updateRequested) {
    return (e) => {
        e.preventDefault();
        updateRequested(value, id);
    };
}
export function isMapping(location, componentMappingToPropertyNames) {
    return (componentMappingToPropertyNames &&
        typeof formItemMapping(componentMappingToPropertyNames, location) === "string");
}
export function getLabel(label, title) {
    return label === "" && title !== void 0 ? title : label;
}
