import FormSection from "./form-section";
import { getActiveComponentAndSection, getBreadcrumbs, getDataCache, getNavigation, isRootLocation, } from "./form.utilities";
import { cloneDeep, get, set, unset } from "lodash-es";
import * as React from "react";
import styles from "./form.style";
import { mapSchemaLocationFromDataLocation } from "@microsoft/fast-data-utilities-react";
import manageJss from "@microsoft/fast-jss-manager-react";
import { mapPluginsToSchema } from "./form-plugin.utilities";
/**
 * Schema form component definition
 * @extends React.Component
 */
class Form extends React.Component {
    constructor(props) {
        super(props);
        this.handleBreadcrumbClick = (schemaLocation, dataLocation, schema) => {
            return (e) => {
                e.preventDefault();
                this.handleUpdateActiveSection(schemaLocation, dataLocation, schema);
            };
        };
        this.handleOnChange = (location, data, isArray, index, isChildren) => {
            let obj = cloneDeep(this.props.data);
            const currentData = location === "" ? obj : get(obj, location);
            if (isArray) {
                let newArray;
                if (typeof index !== "undefined") {
                    newArray = currentData.filter((item, itemIndex) => {
                        return itemIndex !== index;
                    });
                }
                else {
                    newArray = currentData;
                    newArray.push(data);
                }
                location === "" ? (obj = newArray) : set(obj, location, newArray);
            }
            else {
                if (typeof data === "undefined") {
                    location === "" ? (obj = void 0) : unset(obj, location);
                }
                else {
                    location === "" ? (obj = data) : set(obj, location, data);
                }
            }
            if (isChildren) {
                const children = get(obj, location);
                if (Array.isArray(children) && children.length === 1) {
                    set(obj, location, children[0]);
                }
            }
            this.props.onChange(obj);
        };
        /**
         * Handles the form submit
         */
        this.handleSubmit = (e) => {
            e.preventDefault();
        };
        /**
         * Handles an update to the active section and component
         */
        this.handleUpdateActiveSection = (schemaLocation, dataLocation, schema) => {
            if (this.props.location && this.props.location.onChange) {
                this.props.location.onChange(dataLocation);
            }
            else {
                const state = getActiveComponentAndSection(schemaLocation, dataLocation, schema);
                state.navigation = getNavigation(dataLocation || "", this.props.data, this.props.schema, this.props.childOptions);
                this.setState(state);
            }
        };
        this.untitled = "Untitled";
        const schema = mapPluginsToSchema(this.props.schema, this.props.data, this.props.plugins);
        if (JSON.stringify(schema) !== JSON.stringify(this.props.schema)) {
            this.props.onSchemaChange(schema);
        }
        this.state = {
            titleProps: schema && schema.title ? schema.title : this.untitled,
            schema: typeof this.props.plugins !== "undefined" ? schema : this.props.schema,
            activeDataLocation: props.location && typeof props.location === "string"
                ? props.location
                : "",
            dataCache: this.props.data,
            navigation: typeof this.props.location !== "undefined" // Location has been passed
                ? getNavigation(this.props.location.dataLocation, this.props.data, schema, this.props.childOptions)
                : getNavigation("", this.props.data, schema, this.props.childOptions),
        };
    }
    render() {
        return (React.createElement("div", { className: this.getClassNames() },
            React.createElement("form", { onSubmit: this.handleSubmit },
                this.renderBreadcrumbs(),
                this.renderSection())));
    }
    /**
     * React lifecycle hook
     */
    componentWillReceiveProps(nextProps) {
        const state = this.updateStateForNewProps(nextProps, this.props.data !== nextProps.data, JSON.stringify(this.props.schema) !== JSON.stringify(nextProps.schema), this.props.location !== nextProps.location);
        if (state) {
            this.setState(state);
        }
    }
    getClassNames() {
        let classNames = get(this.props, "managedClasses.form", "");
        if (typeof this.props.className === "string") {
            classNames = `${classNames} ${this.props.className}`;
        }
        return classNames;
    }
    /**
     * Update the state when a new schema is given
     */
    /* tslint:disable-next-line */
    updateStateForNewProps(props, updateData, updateSchema, updateLocation) {
        let state = {};
        if (updateData) {
            state = this.getStateWithUpdatedDataCache(props, state);
        }
        if (updateSchema) {
            state = Object.assign({}, state, this.getStateWithUpdatedFormProps(props, state));
        }
        if (updateLocation) {
            state = Object.assign({}, state, this.getStateWithUpdatedLocation(props, state));
        }
        if (typeof props.plugins !== "undefined" &&
            typeof props.onSchemaChange === "function" &&
            (updateData || updateSchema)) {
            const updatedSchema = mapPluginsToSchema(props.schema, props.data, props.plugins);
            if (JSON.stringify(updatedSchema) !== JSON.stringify(props.schema)) {
                props.onSchemaChange(updatedSchema);
            }
        }
        if (updateData || updateSchema || updateLocation) {
            return state;
        }
    }
    /**
     * Gets the state object with an updated data cache
     */
    getStateWithUpdatedDataCache(props, state) {
        const dataCache = typeof this.state !== "undefined" &&
            typeof this.state.dataCache !== "undefined"
            ? this.state.dataCache
            : void 0;
        const schemaState = {
            dataCache: getDataCache(dataCache, props.data),
        };
        return Object.assign({}, state, schemaState);
    }
    /**
     * Gets the state object with updated locations, title and breadcrumbs
     */
    getStateWithUpdatedFormProps(props, state) {
        const schemaState = {
            titleProps: props.schema && props.schema.title ? props.schema.title : this.untitled,
            schema: props.schema,
            activeDataLocation: "",
            dataCache: cloneDeep(props.data),
            navigation: this.getUpdatedNavigation(props, state),
        };
        return Object.assign({}, state, schemaState);
    }
    /**
     * Gets the state with updated location
     */
    getStateWithUpdatedLocation(props, state) {
        const location = props.location
            ? {
                dataLocation: props.location.dataLocation,
                onChange: props.location.onChange,
            }
            : void 0;
        const locationState = {
            activeDataLocation: props.location && props.location.dataLocation
                ? props.location.dataLocation
                : "",
            schema: props.schema,
            location,
            navigation: this.getUpdatedNavigation(props, state),
        };
        return Object.assign({}, state, locationState);
    }
    getUpdatedNavigation(props, state) {
        return getNavigation(props.location ? props.location.dataLocation : state.activeDataLocation || "", props.data, props.schema, props.childOptions);
    }
    /**
     * Generates the breadcrumb navigation
     */
    renderBreadcrumbs() {
        const breadcrumbs = getBreadcrumbs(this.state.navigation, this.handleBreadcrumbClick);
        if (breadcrumbs.length > 1) {
            return (React.createElement("ul", { className: this.props.managedClasses.form_breadcrumbs }, this.renderBreadcrumbItems(breadcrumbs)));
        }
    }
    renderBreadcrumbItems(items) {
        return items.map((item, index) => {
            if (index === items.length - 1) {
                return (React.createElement("li", { key: index },
                    React.createElement("span", null, item.text)));
            }
            return (React.createElement("li", { key: index },
                React.createElement("a", { href: item.href, onClick: item.onClick }, item.text)));
        });
    }
    getData(propKey, location) {
        return isRootLocation(this.state.activeDataLocation)
            ? this[location][propKey]
            : get(this[location][propKey], this.state.activeDataLocation);
    }
    /**
     * Render the section to be shown
     */
    renderSection() {
        return (React.createElement(FormSection, { schema: this.state.navigation[this.state.navigation.length - 1].schema, onChange: this.handleOnChange, onUpdateActiveSection: this.handleUpdateActiveSection, data: this.getData("data", "props"), dataCache: this.getData("dataCache", "state"), schemaLocation: mapSchemaLocationFromDataLocation(this.state.activeDataLocation, this.props.data, this.props.schema), dataLocation: this.state.activeDataLocation, untitled: this.untitled, childOptions: this.props.childOptions, componentMappingToPropertyNames: this.props.componentMappingToPropertyNames, attributeSettingsMappingToPropertyNames: this.props.attributeSettingsMappingToPropertyNames, orderByPropertyNames: this.props.orderByPropertyNames }));
    }
}
Form.displayName = "Form";
export default manageJss(styles)(Form);
