import { cloneDeep, get, mergeWith } from "lodash-es";
import { getChildOptionBySchemaId, getDataLocationsOfChildren, getPartialData, mapSchemaLocationFromDataLocation, normalizeDataLocation, } from "@microsoft/fast-data-utilities-react";
import { reactChildrenStringSchema } from "./form-item.children.text";
const squareBracketsRegex = /\[(\d+?)\]/g;
const propsKeyword = "props";
export var PropertyKeyword;
(function (PropertyKeyword) {
    PropertyKeyword["properties"] = "properties";
    PropertyKeyword["reactProperties"] = "reactProperties";
})(PropertyKeyword || (PropertyKeyword = {}));
/**
 * Gets the data cache based on a new data object and
 * previous data object
 */
export function getDataCache(dataCache, newData) {
    let newDataCache = typeof dataCache !== "undefined" ? cloneDeep(dataCache) : newData;
    if (newDataCache !== newData) {
        newDataCache = mergeWith(dataCache, newData, cachedArrayResolver);
    }
    return newDataCache;
}
/**
 * Resolves arrays when the object is an array and the data passed is an array
 */
function cachedArrayResolver(objValue, srcValue) {
    if (Array.isArray(objValue) && Array.isArray(srcValue)) {
        return srcValue;
    }
}
/**
 * Determines the navigation from
 * - section links
 * - child components
 * - array items
 * - breadcrumb links
 */
export function getActiveComponentAndSection(schemaLocation, dataLocation, schema) {
    const state = {};
    state.activeDataLocation = dataLocation;
    state.activeSchemaLocation = schemaLocation;
    // if the schema is undefined its most likely an array
    if (typeof schema !== "undefined") {
        state.schema = schema;
        state.titleProps = schema.title || "Untitled";
    }
    return state;
}
/**
 * Gets locations from individual location segments
 * Example:
 * getLocationsFromSegments(["children[0].props.object"])
 * output: ["children[0]", "children[0].props", "children[0].props.object"]
 */
export function getLocationsFromSegments(segments) {
    return segments.map((location, index) => {
        return segments.slice(0, index + 1).join(".");
    });
}
/**
 * Gets the navigational items
 */
export function getNavigation(dataLocation, data, schema, childOptions) {
    const allChildOptions = getReactDefaultChildren().concat(childOptions);
    const dataLocationsOfChildren = getDataLocationsOfChildren(schema, data, allChildOptions);
    const normalizedDataLocation = !dataLocationsOfChildren.includes(dataLocation)
        ? normalizeDataLocation(dataLocation, data)
        : typeof get(data, dataLocation) === "string"
            ? normalizeDataLocation(dataLocation, data)
            : `${normalizeDataLocation(dataLocation, data)}.${propsKeyword}`;
    const dataLocations = new Set([""].concat(getLocationsFromSegments(normalizedDataLocation.split("."))));
    const navigationItems = [];
    let currentComponentSchema = schema;
    let lastComponentDataLocation = "";
    dataLocations.forEach((dataLocationItem) => {
        if (dataLocationsOfChildren.includes(dataLocationItem)) {
            const isChildString = typeof get(data, dataLocationItem) === "string";
            currentComponentSchema = getSchemaByDataLocation(schema, data, dataLocationItem, allChildOptions);
            lastComponentDataLocation = isChildString
                ? dataLocationItem
                : `${dataLocationItem}.${propsKeyword}`;
            if (isChildString) {
                navigationItems.push(getNavigationItem(dataLocationItem, "", reactChildrenStringSchema, data));
            }
        }
        else {
            const isRoot = isRootLocation(lastComponentDataLocation);
            const rootLocationOfComponent = isRoot
                ? ""
                : lastComponentDataLocation.replace(dataLocationItem, "");
            const dataLocationFromLastComponent = getCurrentComponentDataLocation(dataLocationItem, lastComponentDataLocation);
            let currentSchemaLocation = mapSchemaLocationFromDataLocation(isRoot ? dataLocationItem : dataLocationFromLastComponent, isRoot ? data : get(data, rootLocationOfComponent), currentComponentSchema);
            const currentSchemaLocationSegments = currentSchemaLocation.split(".");
            const currentSchemaLocationSegmentsLength = currentSchemaLocationSegments.length;
            if (!isNaN(parseInt(currentSchemaLocationSegments[currentSchemaLocationSegmentsLength - 1], 10)) &&
                (currentSchemaLocationSegments[currentSchemaLocationSegmentsLength - 2] === "oneOf" ||
                    currentSchemaLocationSegments[currentSchemaLocationSegmentsLength - 2] === "anyOf")) {
                currentSchemaLocation = currentSchemaLocationSegments
                    .slice(0, currentSchemaLocationSegmentsLength - 2)
                    .join(".");
            }
            const currentSchema = dataLocationFromLastComponent === ""
                ? currentComponentSchema
                : get(currentComponentSchema, currentSchemaLocation);
            navigationItems.push(getNavigationItem(dataLocationItem, currentSchemaLocation, currentSchema, data));
        }
    });
    return navigationItems;
}
/**
 * Get a single navigation item
 */
export function getNavigationItem(dataLocation, schemaLocation, schema, data) {
    return {
        dataLocation,
        schemaLocation,
        title: schema.title || "Untitled",
        data: getPartialData(dataLocation, data),
        schema,
    };
}
/**
 * Get React's default children
 */
export function getReactDefaultChildren() {
    return [
        {
            name: "Text",
            component: null,
            schema: reactChildrenStringSchema,
        },
    ];
}
/**
 * Gets the data location from the current component
 */
export function getCurrentComponentDataLocation(dataLocation, lastComponentDataLocation) {
    return dataLocation.replace(lastComponentDataLocation, "").replace(/^\./, "");
}
/**
 * Removes any references to array index
 */
export function normalizeSchemaLocation(schemaLocation) {
    return schemaLocation.replace(squareBracketsRegex, "");
}
/**
 * Check to see if we are on the root location
 */
export function isRootLocation(location) {
    return location === "";
}
/**
 * Gets breadcrumbs from navigation items
 */
export function getBreadcrumbs(navigation, handleClick) {
    return navigation.map((navigationItem) => {
        return {
            href: navigationItem.dataLocation,
            text: navigationItem.title,
            onClick: handleClick(navigationItem.schemaLocation, navigationItem.dataLocation, navigationItem.schema),
        };
    });
}
/**
 * Finds the schema using the data location
 */
export function getSchemaByDataLocation(currentSchema, data, dataLocation, childOptions) {
    if (dataLocation === "") {
        return currentSchema;
    }
    const subData = get(data, dataLocation);
    const id = subData ? subData.id : void 0;
    const childOptionWithMatchingSchemaId = childOptions.find((childOption) => {
        return childOption.schema.id === id;
    });
    return childOptionWithMatchingSchemaId
        ? childOptionWithMatchingSchemaId.schema
        : currentSchema;
}
/**
 * Finds the component using the schema id
 */
export function getComponentByDataLocation(id, childOptions) {
    const childOption = getChildOptionBySchemaId(id, childOptions);
    return childOption ? childOption.component : null;
}
